/*
** pork_proto.c
** Copyright (C) 2003 Ryan McCabe <ryan@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include <pork.h>
#include <pork_missing.h>
#include <pork_util.h>
#include <pork_list.h>
#include <pork_proto.h>

static struct pork_proto *proto_table[PROTO_MAX];

inline struct pork_proto *proto_get(u_int32_t protocol) {
	if (protocol < PROTO_MAX)
		return (proto_table[protocol]);

	return (NULL);
}

int proto_init(u_int32_t protocol, int (*init_func)(struct pork_proto *)) {
	struct pork_proto *new_proto;
	int ret;

	if (protocol >= PROTO_MAX ||
		proto_table[protocol] != NULL || init_func == NULL)
	{
		return (-1);
	}

	new_proto = xcalloc(1, sizeof(*new_proto));
	new_proto->protocol = protocol;

	ret = init_func(new_proto);
	if (ret != 0) {
		free(new_proto);
		return (ret);
	}

	proto_table[protocol] = new_proto;
	return (0);
}

void proto_destroy(void) {
	size_t i;

	for (i = 0 ; i < PROTO_MAX ; i++)
		free(proto_table[i]);
}
